﻿using System;
using System.Collections.Generic;
using System.Configuration;
using System.IO;
using System.Linq;
using System.Threading.Tasks;
using System.Windows.Media;
using Buddy.Coroutines;
using ff14bot;
using ff14bot.AClasses;
using ff14bot.Behavior;
using ff14bot.Enums;
using ff14bot.Helpers;
using ff14bot.Managers;
using ff14bot.RemoteWindows;
using Newtonsoft.Json;
using TreeSharp;

namespace MedicineGuyPlugin
{
    // ReSharper disable once UnusedMember.Global
    public class MedicineGuyPlugin : BotPlugin
    {
        #region Properties

        private static bool HasMedicineBuff
        {
            get { return Core.Me.HasAura(49); }
        }

        #endregion

        #region Drink Medicine Logic

        private static async Task<bool> DrinkMedicine()
        {
            if (string.IsNullOrEmpty(Settings.Instance.MedicineName))
            {
                Logging.Write(Colors.Aqua, "[MedicineGuyPlugin] Please select a new Medicine Item from the settings window.");
                return false;
            }

            var medicineSetting = Settings.Instance.MedicineName;
            var index = medicineSetting.IndexOf('(');
            var medicine = index > 0 ? medicineSetting.Substring(0, index) : medicineSetting;

            var item =
                InventoryManager.FilledSlots.InventoryItems()
                    .MedicineItems()
                    .FirstOrDefault(x => x.Item.EnglishName == medicine && x.CanUse());

            if (item == null) return false;

            if (FishingManager.State != FishingState.None)
            {
                Actionmanager.DoAction(299, Core.Me);
                await Coroutine.Wait(5000, () => FishingManager.State == FishingState.None);
            }

            if (GatheringManager.WindowOpen)
            {
                Logging.Write(Colors.Aqua, "[MedicineGuy] Currently Gathering Items, waiting until we are done.");
                return false;
            }

            if (CraftingManager.IsCrafting)
            {
                Logging.Write(Colors.Pink, "[MedicineGuy] Currently Crafting Items, waiting until we are done.");
                return false;
            }

            if (CraftingLog.IsOpen)
            {
                Logging.Write(Colors.Pink, "[MedicineGuy] Closing Crafting Log so we can drink medicine.");
                CraftingLog.Close();
            }

            if (Core.Me.IsMounted)
            {
                Logging.Write(Colors.Aqua, "[MedicineGuy] Dismounting so we can drink medicine.");
                await CommonTasks.StopAndDismount();
            }

            Logging.Write(Colors.Aqua, "[MedicineGuy] Now drinking " + item.Item.EnglishName);
            item.UseItem();
            await Coroutine.Wait(5000, () => HasMedicineBuff);

            return true;
        }

        #endregion

        #region Fields

        private Composite _coroutine;
        private Gui _settingsForm;

        #endregion

        #region Required Overrides

        public override string Author
        {
            get { return "Cloud30000"; }
        }

        public override string Name
        {
            get { return "MedicineGuy"; }
        }

        public override Version Version
        {
            get { return new Version(1, 0, 0, 0); }
        }

        #endregion

        #region Optional Overrides

        public override void OnInitialize()
        {
            _coroutine = new Decorator(c => !HasMedicineBuff, new ActionRunCoroutine(r => DrinkMedicine()));
        }

        public override void OnEnabled()
        {
            TreeRoot.OnStart += OnBotStart;
            TreeRoot.OnStop += OnBotStop;
            TreeHooks.Instance.OnHooksCleared += OnOnHooksCleared;

            if (TreeRoot.IsRunning)
            {
                AddHooks();
            }
        }

        public override void OnDisabled()
        {
            TreeRoot.OnStart -= OnBotStart;
            TreeRoot.OnStop -= OnBotStop;
            TreeHooks.Instance.OnHooksCleared -= OnOnHooksCleared;
            RemoveHooks();
        }

        public override void OnShutdown()
        {
            TreeRoot.OnStart -= OnBotStart;
            TreeRoot.OnStop -= OnBotStop;
            TreeHooks.Instance.OnHooksCleared -= OnOnHooksCleared;
            RemoveHooks();
        }

        public override bool WantButton
        {
            get { return true; }
        }

        public override void OnButtonPress()
        {
            if (_settingsForm == null || _settingsForm.IsDisposed || _settingsForm.Disposing)
                _settingsForm = new Gui();

            _settingsForm.ShowDialog();
        }

        #endregion

        #region Hook Logic

        private void AddHooks()
        {
            Logging.Write(Colors.Aqua, "Adding MedicineGuy Hook");
            TreeHooks.Instance.InsertHook("TreeStart", 0, _coroutine);
        }

        private void RemoveHooks()
        {
            Logging.Write(Colors.Aqua, "Removing MedicineGuy Hook");
            TreeHooks.Instance.RemoveHook("TreeStart", _coroutine);
        }

        #endregion

        #region Event Handlers

        private void OnBotStop(BotBase bot)
        {
            RemoveHooks();
        }

        private void OnBotStart(BotBase bot)
        {
            AddHooks();
        }

        private void OnOnHooksCleared(object sender, EventArgs e)
        {
            AddHooks();
        }

        #endregion
    }

    #region Extensions Class

    public static class Extensions
    {
        public static IEnumerable<BagSlot> MedicineItems(this IEnumerable<BagSlot> inventory)
        {
            return inventory.Where(x => x.MedicineItem());
        }

        public static IEnumerable<BagSlot> InventoryItems(this IEnumerable<BagSlot> inventory)
        {
            return
                inventory.Where(
                    x =>
                        x.BagId == InventoryBagId.Bag1 || x.BagId == InventoryBagId.Bag2 ||
                        x.BagId == InventoryBagId.Bag3 || x.BagId == InventoryBagId.Bag4);
        }

        private static bool MedicineItem(this BagSlot slot)
        {
            return (slot.Item.EquipmentCatagory == ItemUiCategory.Medicine) && (int)slot.Item.ItemRole == 6;
        }
    }

    #endregion

    #region Settings Class

    public class Settings : JsonSettings
    {
        [JsonIgnore]
        private static Settings _instance;
        // ReSharper disable once UnusedParameter.Local
        // ReSharper disable once MemberCanBePrivate.Global
        public Settings(string filename)
            : base(Path.Combine(CharacterSettingsDirectory, "MedicineGuy.json"))
        {
        }

        public static Settings Instance
        {
            get { return _instance ?? (_instance = new Settings("Settings")); }
        }

        [Setting]
        public string MedicineName { get; set; }
    }

    #endregion
}